/* ---- msp430g2231 ADC10 TestCode ----------------------------------- #### (c) B. Nollmeyer 10/2010 #### ---- */
/*                                                                                                             */

/*
   ADC Oversampling 12 BIT, Gain, Offset Error Kompensation mittels "Fixed-point Arithmetik".
   AREF wird auf die interne 2,5V Referenz eingestellt. Messung des Temperatur Sensor, Ausgabe in Grad Celsius.
   Die Temperaturmessung wird ber 2 Referenzpunkte linearisiert.
   ---- Code 694 Byte ----
*/

#include "msp430g2231.h"
#include "inttypes.h"

// Referenzpunkte ADC
#define AdcRawHigh 1856
#define AdcRawLow  1756
#define DeltaRaw (AdcRawHigh - AdcRawLow)

// Skalierung Temperatur in Celsius
#define RefValHigh 370.0                                          // Ausgabewert fr AdcRawHigh = 37 Grad Celsius
#define RefValLow   10.0                                          // Ausgabewert fr AdcRawLow  =  1 ""
#define Scale      65536                                          // zur Basis 2^16
#define DeltaOut (RefValHigh - RefValLow)

// Berechnung der ADC Korrekturwerte mittels Referenzpunkte
#define Slope     (DeltaOut / DeltaRaw)                           // ADC Steigung
#define zeroPoint (RefValLow - AdcRawLow * Slope)
#define Offset    (int32_t) (zeroPoint * Scale)                   // ADC Offset Fehler zur Basis 2^16
#define Factor    (int32_t) (Slope * Scale)                       // ADC Faktor zur Basis 2^16

#define NoOfLeadZero 3

#define LED1     BIT0                                             // rote Led
#define LED2     BIT6                                             // grne Led
#define TXD      BIT1                                             // TXD an P1.1
#define AdcInput 0x10                                             // ADC Eingang an P1.6
#define BitTime   104                                             // 9600 Baud, SMCLK=1MHz (1MHz/9600)=104

uint8_t BitCnt;                                                   // Bit count fr die serielle Ausgabe
char TXByte;                                                      // TXByte via UART when Transmit() is called

// Prototypes
void Transmit (void);
void putChar (char c);
void AdcValOut (int16_t AdcValue);
int16_t getAdcValue (void);

void main (void)
{
    WDTCTL = WDTPW + WDTHOLD;                                     // Stop watchdog timer
    BCSCTL1 = CALBC1_1MHZ;
    DCOCTL = CALDCO_1MHZ;                                         // SMCLK = DCO = 1MHz
    P1SEL |= TXD;
    P1DIR |= LED1 + LED2 + TXD;                                   // Led1, Led2, TXD als Ausgang setzen
    ADC10CTL1 |= INCH_10;                                         // ADC IN an Temperatur Sensor
    ADC10CTL0 = SREF_1 + ADC10SHT_2 + REF2_5V + REFON + ADC10ON;  // SHT 16x ADC10CLKs, VREF 2.5V, ADC ON
    P1OUT &= ~ LED1;

    __bis_SR_register(GIE);                                       // interrupts enabled

    for (;;) {
        AdcValOut (getAdcValue ());
        P1OUT ^= LED2;
        __delay_cycles (500000);                                  // ~ 2x pro Sekunde
    }
}

// adc loop
int16_t getAdcValue (void)
{
    uint16_t AvgCount = 0;
    int32_t AvgSum = 0;

    for (AvgCount = 0; AvgCount < 512; AvgCount ++) {
        ADC10CTL0 &= ~ADC10IFG;
        ADC10CTL0 |= ENC + ADC10SC;                               // AD Wandler starten
        while (!(ADC10CTL0 & ADC10IFG));                          // AD Wandlung fertig ?
        AvgSum += ADC10MEM;                                       // Mittelwert ber 512 Messungen
    }
    return ((((AvgSum >> 7) * Factor) + Offset) >> 16);           // skaliert auf 12 BIT
}

// ADC Value integer to ASCII
void AdcValOut (int16_t AdcValue)
{
    uint8_t AdcValArray [5];                                      // Integer Array
    uint8_t m = 3;                                                // Schleifenzhler fr Ausgabe

    if (AdcValue < 0) {
        AdcValue = -AdcValue;                                     // * -1
        putChar ('-');                                            // Vorzeichen setzen
    } else {
        putChar ('+');
    }
    do  {
        AdcValArray [m] = '0' + AdcValue %10;                     // Berechnung der einzelnen Stellen sowie
        AdcValue /= 10;                                           // Wandlung nach ASCII und abspeichern im
    } while (m--);                                                // Array

    for (m = 0; m < NoOfLeadZero; m++) {                          // fhrende Nullen entfernen
        if (AdcValArray [m] != '0') {
            break;
        } else {
            AdcValArray [m] = ' ';
        }
    }
    m = 0;
    while (m != 3) {                                              // Anzahl der auszugebenden Stellen = 2
        putChar (AdcValArray [++m]);                              // ADC Wert ausgeben
        if (m == 2) break;
    }
    putChar (' ');
    putChar ('C');
    putChar ('\r');
}

// simple putChar
void putChar (char c)
{
    TXByte = c;
    Transmit ();
}

// TXing a byte
void Transmit (void)
{
    CCTL0 = OUT;                                                  // TXD Idle as Mark
    TACTL = TASSEL_2 + MC_2;                                      // SMCLK, continuous mode
    BitCnt = 0xA;                                                 // Load Bit counter, 8 bits + ST/SP
    CCR0 = TAR;
    CCR0 += BitTime;                                              // Set time till first bit
    TXByte |= 0x100;                                              // Add stop bit to TXByte (which is logical 1)
    TXByte <<= 1;                                                 // Add start bit (which is logical 0)
    CCTL0 =  CCIS0 + OUTMOD0 + CCIE;                              // Set signal, intial value, enable interrupts
    while (CCTL0 & CCIE);                                         // Wait for TX completion
    TACTL = TASSEL_2;                                             // SMCLK, timer off (for power consumption)
}

// Timer A0 interrupt service routine
#pragma vector=TIMERA0_VECTOR
__interrupt void Timer_A (void)
{
    CCR0 += BitTime;                                              // Add Offset to CCR0
    if (BitCnt == 0) {                                            // If all bits TXed, disable interrupt
        CCTL0 &= ~ CCIE ;
    } else {
        CCTL0 |= OUTMOD2;                                         // TX Space
        if (TXByte & 0x01)
            CCTL0 &= ~ OUTMOD2;                                   // TX Mark
        TXByte >>= 1;
        BitCnt --;
    }
}